package com.sun.corba.se.impl.presentation.rmi;

import java.rmi.Remote;
import com.sun.corba.se.impl.orbutil.graph.Node;
import com.sun.corba.se.impl.orbutil.graph.GraphImpl;
import com.sun.corba.se.spi.orbutil.proxy.InvocationHandlerFactory;
import com.sun.corba.se.spi.presentation.rmi.IDLNameTranslator;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.HashSet;
import java.util.Set;
import com.sun.corba.se.impl.orbutil.graph.Graph;
import javax.rmi.CORBA.Tie;
import com.sun.corba.se.spi.presentation.rmi.DynamicMethodMarshaller;
import java.lang.reflect.Method;
import java.util.HashMap;
import com.sun.corba.se.impl.logging.ORBUtilSystemException;
import java.util.Map;
import com.sun.corba.se.spi.presentation.rmi.PresentationManager;

public final class PresentationManagerImpl implements PresentationManager
{
    private Map classToClassData;
    private Map methodToDMM;
    private PresentationManager.StubFactoryFactory staticStubFactoryFactory;
    private PresentationManager.StubFactoryFactory dynamicStubFactoryFactory;
    private ORBUtilSystemException wrapper;
    private boolean useDynamicStubs;
    
    public PresentationManagerImpl(boolean useDynamicStubs) {
        this.wrapper = null;
        this.useDynamicStubs = useDynamicStubs;
        this.wrapper = ORBUtilSystemException.get("rpc.presentation");
        this.classToClassData = new HashMap();
        this.methodToDMM = new HashMap();
    }
    
    public synchronized DynamicMethodMarshaller getDynamicMethodMarshaller(Method method) {
        if (method == null) {
            return null;
        }
        DynamicMethodMarshaller dynamicMethodMarshaller = (Object)this.methodToDMM.get(method);
        if (dynamicMethodMarshaller == null) {
            dynamicMethodMarshaller = new DynamicMethodMarshallerImpl(method);
            this.methodToDMM.put(method, dynamicMethodMarshaller);
            return dynamicMethodMarshaller;
        }
        return dynamicMethodMarshaller;
    }
    
    public synchronized PresentationManager.ClassData getClassData(Class clazz) {
        PresentationManager.ClassData classData = (Object)this.classToClassData.get(clazz);
        if (classData == null) {
            classData = new PresentationManagerImpl.ClassDataImpl(clazz);
            this.classToClassData.put(clazz, classData);
            return classData;
        }
        return classData;
    }
    
    public PresentationManager.StubFactoryFactory getStubFactoryFactory(boolean b) {
        if (b) {
            return this.dynamicStubFactoryFactory;
        }
        return this.staticStubFactoryFactory;
    }
    
    public void setStubFactoryFactory(boolean b, PresentationManager.StubFactoryFactory stubFactoryFactory) {
        if (b) {
            this.dynamicStubFactoryFactory = stubFactoryFactory;
        }
        else {
            this.staticStubFactoryFactory = stubFactoryFactory;
        }
    }
    
    public Tie getTie() {
        return this.dynamicStubFactoryFactory.getTie(null);
    }
    
    public boolean useDynamicStubs() {
        return this.useDynamicStubs;
    }
    
    private Set getRootSet(Class superclass, PresentationManagerImpl.NodeImpl nodeImpl, Graph graph) {
        if (superclass.isInterface()) {
            graph.add(nodeImpl);
            return graph.getRoots();
        }
        HashSet<Object> set = new HashSet();
        while (superclass != null && !superclass.equals(Object.class)) {
            PresentationManagerImpl.NodeImpl o = new NodeImpl(superclass);
            graph.add(o);
            set.add(o);
            superclass = superclass.getSuperclass();
        }
        graph.getRoots();
        graph.removeAll(set);
        return graph.getRoots();
    }
    
    private Class[] getInterfaces(Set set) {
        Class[] array = new Class[set.size()];
        Iterator<Object> iterator = set.iterator();
        int n = 0;
        while (iterator.hasNext()) {
            array[n++] = ((PresentationManagerImpl.NodeImpl)iterator.next()).getInterface();
        }
        return array;
    }
    
    private String[] makeTypeIds(PresentationManagerImpl.NodeImpl nodeImpl, Graph c, Set set) {
        HashSet set2 = new HashSet(c);
        set2.removeAll(set);
        ArrayList<String> list = new ArrayList();
        if (set.size() > 1) {
            list.add(nodeImpl.getTypeId());
        }
        this.addNodes(list, set);
        this.addNodes(list, set2);
        return (String[])(String[])list.toArray(new String[list.size()]);
    }
    
    private void addNodes(List list, Set set) {
        Iterator<Object> iterator = set.iterator();
        while (iterator.hasNext()) {
            list.add(((PresentationManagerImpl.NodeImpl)iterator.next()).getTypeId());
        }
    }
    
    static Set access$000(PresentationManagerImpl presentationManagerImpl, Class clazz, PresentationManagerImpl.NodeImpl nodeImpl, Graph graph) {
        return presentationManagerImpl.getRootSet(clazz, nodeImpl, graph);
    }
    
    static Class[] access$100(PresentationManagerImpl presentationManagerImpl, Set set) {
        return presentationManagerImpl.getInterfaces(set);
    }
    
    static String[] access$200(PresentationManagerImpl presentationManagerImpl, PresentationManagerImpl.NodeImpl nodeImpl, Graph graph, Set set) {
        return presentationManagerImpl.makeTypeIds(nodeImpl, graph, set);
    }
    
    private class ClassDataImpl implements PresentationManager.ClassData
    {
        private Class cls;
        private IDLNameTranslator nameTranslator;
        private String[] typeIds;
        private PresentationManager.StubFactory sfactory;
        private InvocationHandlerFactory ihfactory;
        private Map dictionary;
        
        public ClassDataImpl(Class cls) {
            this.cls = cls;
            GraphImpl graphImpl = new GraphImpl();
            PresentationManagerImpl.NodeImpl nodeImpl = new NodeImpl(cls);
            Set<Object> access$000 = PresentationManagerImpl.access$000(PresentationManagerImpl.this, cls, nodeImpl, graphImpl);
            this.nameTranslator = IDLNameTranslatorImpl.get(PresentationManagerImpl.access$100(PresentationManagerImpl.this, access$000));
            this.typeIds = PresentationManagerImpl.access$200(PresentationManagerImpl.this, nodeImpl, graphImpl, access$000);
            this.ihfactory = new InvocationHandlerFactoryImpl(PresentationManagerImpl.this, this);
            this.dictionary = new HashMap();
        }
        
        public Class getMyClass() {
            return this.cls;
        }
        
        public IDLNameTranslator getIDLNameTranslator() {
            return this.nameTranslator;
        }
        
        public String[] getTypeIds() {
            return this.typeIds;
        }
        
        public InvocationHandlerFactory getInvocationHandlerFactory() {
            return this.ihfactory;
        }
        
        public Map getDictionary() {
            return this.dictionary;
        }
    }
    
    private static class NodeImpl implements Node
    {
        private Class interf;
        
        public Class getInterface() {
            return this.interf;
        }
        
        public NodeImpl(Class interf) {
            this.interf = interf;
        }
        
        public String getTypeId() {
            return "RMI:" + this.interf.getName() + ":0000000000000000";
        }
        
        public Set getChildren() {
            HashSet<PresentationManagerImpl.NodeImpl> set = new HashSet();
            Class[] interfaces = this.interf.getInterfaces();
            for (int i = 0; i < interfaces.length; ++i) {
                Class<Object> obj = interfaces[i];
                if (Remote.class.isAssignableFrom(obj) && !Remote.class.equals(obj)) {
                    set.add(new NodeImpl(obj));
                }
            }
            return set;
        }
        
        @Override
        public String toString() {
            return "NodeImpl[" + this.interf + "]";
        }
        
        @Override
        public int hashCode() {
            return this.interf.hashCode();
        }
        
        @Override
        public boolean equals(Object o) {
            return this == o || (o instanceof PresentationManagerImpl.NodeImpl && ((PresentationManagerImpl.NodeImpl)o).interf.equals(this.interf));
        }
    }
}
